##Node summarizer for trees obtained in a multi-tree lagrange

#written by Jeremy M. Beaulieu

##INPUT##

#key.trees are the set of key.tre output from lagrange (ex. Camp.1000.TIGHT.2state.key.tre)
#taxa1 and taxa2 are the two taxa whose MRCA is the node of interest (ex. "Ilex", "Echinops")
#file.string is the name of the output file from lagrange (ex. Camp_2State.txt)

require(geiger)

truncAIC<-function(key.trees, taxa1, taxa2, file.string){

	cat("Getting node numbers from key", "\n")

	node.list<-NodeAge(key.trees, taxa1, taxa2)
	write.table(node.list, "tmp", quote=FALSE, col.names=FALSE, row.names=FALSE)

	cat("Finished. Getting reconstructions from lagrange output", "\n")
	
	system(paste('ruby results_parser.rb', file.string, 'tmp', sep=" "))
	system("cat node_data.txt | tr -d '()' > node_data_fixed.txt")
	node.data<-read.delim("node_data_fixed.txt",h=F)
	system(paste("rm node_data_fixed.txt node_data.txt tmp"))

	cat("Finished. Calculating truncated AIC weights", "\n")
	
	node.data[,4]<- -1*node.data[,4]
	pp<-sumRanges(node.data, np=2, key.trees)
	pp[1:5,]
}


##This function is really shittily written, but it works. I plan on cleaning this up eventually (or not):
sumRanges<-function(GIANTS, np=2, key.trees){
	
	AIC <- -2*GIANTS[,4]+2*np
	file1<-cbind(GIANTS,AIC)
	file1[,1]<-as.factor(file1[,1])
	
	for(i in levels(file1[,1])){
		subData<-subset(file1,(file1[,1]%in%i))
		subData<-cbind(subData,min(file1[,5][file1[,1]==i]))
		write.table(subData,file="Data.xls", quote=FALSE, append=TRUE,row.names=FALSE, col.names=FALSE,sep="\t")
	}
	
	mm<-read.delim("Data.xls",h=F)
	string1<-"rm"
	string2<-paste(string1,"Data.xls",sep=" ")
	system(string2)
	
	deltaAIC<-mm[,5]-mm[,6]
	mm<-cbind(mm,deltaAIC)
	RelLikelihood<-exp(deltaAIC*-0.5)
	mm<-cbind(mm,RelLikelihood)
	mm[,1]<-as.factor(mm[,1])
	
	for(i in levels(mm[,1])){
		subData<-subset(mm,(mm[,1]%in%i))
		subData<-cbind(subData,sum(mm[,8][mm[,1]==i]))
		write.table(subData,file="Sums.xls", quote=FALSE, append=TRUE,row.names=FALSE, col.names=FALSE,sep="\t")
	}
	
	FINAL<-read.delim("Sums.xls",h=F)
	string1<-"rm"
	string2<-paste(string1,"Sums.xls",sep=" ")
	system(string2)
	
	AICw<-FINAL[,8]/FINAL[,9]
	FINAL<-cbind(FINAL,AICw)
	
	biogeo<-length(unique(FINAL[,2]))
	res<-matrix(,nrow=biogeo,ncol=2)
	RANGES<-as.numeric(FINAL[,2])
	for(i in 1:biogeo){
		options(scipen=100) 
		res[i,2]<-sum(FINAL[,10][RANGES==i])/length(key.trees)
		
	}
	res[,1]<-sort(as.character(unique(mm[,2])))
	res<-as.data.frame(res)
	res<-res[order(res[,2], decreasing=TRUE),]
	colnames(res)<-c("range","weight")
	res
	
}

##UTILITY FUNCTIONS##
NodeAge<-function(phy, taxa1, taxa2){
	Age<-matrix(nrow=length(phy), ncol=1)
	for (i in 1:length(phy)){
		NODE<-mrca(phy[[i]])[taxa1, taxa2]
#		parent<-phy[[i]]$edge[,1][phy[[i]]$edge[,2]==NODE-1]
		Age[i,1]<-nodeLABEL(phy[[i]])[as.character(NODE)]
	}
	return(Age)	
}

branchTIME<-function(phy){
   
    n<-length(phy$tip.label)
    N<-dim(phy$edge)[1]
    xx<-numeric(phy$Nnode)
    interns<-which(phy$edge[,2]>n)
    for (i in interns) xx[phy$edge[i,2]-n]<-xx[phy$edge[i,1]-n]+phy$edge.length[i]
    depth<-xx[phy$edge[N,1]-n]+phy$edge.length[N]
	xx<-depth-xx
	names(xx)<-(n+1):(n+phy$Nnode)
	xx
}

nodeLABEL<-function(phy){
    n<-length(phy$tip.label)
	xx<-numeric(phy$Nnode)
	xx<-as.matrix(phy$node.label)
    names(xx)<-(n+1):(n+phy$Nnode)
    xx
}

